#include "ltr/include/ltr34api.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#endif



#define LTR34_SEND_SIZE 2560

typedef struct
{
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

static int f_get_params(int argc, char** argv, t_open_param* par)
{
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = NULL;
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3)
    {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4)
        {
            fprintf(stderr, "Неверный формат адреса сервера!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++)
        {
            if ((a[i]<0) || (a[i] > 255))
            {
                fprintf(stderr, "Неверный формат адреса сервера!!\n");
                err = -1;
            }
        }

        if (!err)
        {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}


static int f_send_data(TLTR34 *hnd)
{
    double snd_array[LTR34_SEND_SIZE];
    static DWORD proc_array[LTR34_SEND_SIZE];
    static DWORD snd_size = LTR34_SEND_SIZE;
    int i, err=0;
    int sent;
    for (i=0; i < LTR34_SEND_SIZE; i++) {
        snd_array[i] = 10*sin(2.*3.14*i/(LTR34_SEND_SIZE));
    }

    err = LTR34_ProcessData(hnd, snd_array, proc_array, snd_size, 1);
    if (!err)
    {
        sent = LTR34_Send(hnd, proc_array, snd_size, 25000);
        if (sent < 0)
            err = sent;
        else if (sent > 0)
            printf("sent %d\n", sent);
    }

#if 0
    err = LTR34_Recv(hnd,  proc_array, NULL, snd_size, 5000);
    if (err!=snd_size)
    {
        break;
    }
#endif
    return err;
}



int main(int argc, char** argv)
{
    INT err = LTR_OK;
    TLTR34 hltr34;
    t_open_param par;
#ifdef _WIN32
    /* для вывода русских букв в консоль для ОС Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    LTR34_Init(&hltr34);

    err = LTR34_Open(&hltr34, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
    if (err != LTR_OK)
    {
        fprintf(stderr, "Не удалось установить соединение с модулем. Ошибка %d: %s\n",
                err, LTR34_GetErrorString(err));
    }
    else
    {
        printf("Модуль открыт успешно. Информация о модуле:\n  Название %s\n  Серийный %s\n  Версия ПЛИС %s\n  Максимальное число каналов - %d\n",
               hltr34.ModuleInfo.Name, hltr34.ModuleInfo.Serial, hltr34.ModuleInfo.FPGA_Version, hltr34.ModuleInfo.MaxChannelQnt);
        fflush(stdout);

        hltr34.ChannelQnt = 1;
        hltr34.LChTbl[0] = LTR34_CreateLChannel(1, 0);
        hltr34.FrequencyDivisor = 60;
        hltr34.AcknowledgeType = LTR34_ACKTYPE_STATUS;
        hltr34.RingMode = 0;
        hltr34.ExternalStart = 0;

        err = LTR34_Reset(&hltr34);
        if (err)
        {
            fprintf(stderr, "Не удалось выполнить сброс модуля LTR34. Ошибка %d: %s\n",
                    err, LTR34_GetErrorString(err));
        }
        if (!err)
        {
            err = LTR34_Config(&hltr34);
            if (err)
            {
                fprintf(stderr, "Не удалось сконфигурировать модуль LTR34. Ошибка %d: %s\n",
                        err, LTR34_GetErrorString(err));
            }
            else{
                printf("Настройки записаны успешно!\n");
            }

        }


#if 1
        if (!err)
        {
            int i;
            for (i=0; (i < 10) && !err; i++)
                err = f_send_data(&hltr34);

            if (!err)
                err = LTR34_DACStart(&hltr34);

            //for (i=0; i < 100; i++)
            for (;!err;)
            {
                err = f_send_data(&hltr34);

            }

            LTR34_DACStop(&hltr34);
        }
#endif



        LTR34_Close(&hltr34);
    }
    return err;
}
