unit ltr212api;
interface
uses windows, SysUtils, ltrapi, ltrapitypes;
const
    //  .  . - LRT212_GetErrorString()
    LTR212_NO_ERR=0;
    LTR212_ERR_INVALID_DESCR=               -2001;
    LTR212_ERR_INVALID_CRATE_SN=            -2002;
    LTR212_ERR_INVALID_SLOT_NUM=            -2003;
    LTR212_ERR_CANT_INIT=                   -2004;
    LTR212_ERR_CANT_OPEN_CHANNEL=           -2005;
    LTR212_ERR_CANT_CLOSE=                  -2006;
    LTR212_ERR_CANT_LOAD_BIOS=              -2007;
    LTR212_ERR_CANT_SEND_COMMAND=           -2008;
    LTR212_ERR_CANT_READ_EEPROM=            -2009;
    LTR212_ERR_CANT_WRITE_EEPROM=           -2010;
    LTR212_ERR_CANT_LOAD_IIR=               -2011;
    LTR212_ERR_CANT_LOAD_FIR=               -2012;
    LTR212_ERR_CANT_RESET_CODECS=           -2013;
    LTR212_ERR_CANT_SELECT_CODEC=           -2014;
    LTR212_ERR_CANT_WRITE_REG=              -2015;
    LTR212_ERR_CANT_READ_REG=               -2016;
    LTR212_ERR_WRONG_ADC_SETTINGS=          -2017;
    LTR212_ERR_WRONG_VCH_SETTINGS=          -2018;
    LTR212_ERR_CANT_SET_ADC=                -2019;
    LTR212_ERR_CANT_CALIBRATE=              -2020;
    LTR212_ERR_CANT_START_ADC=              -2021;
    LTR212_ERR_INVALID_ACQ_MODE=            -2022;
    LTR212_ERR_CANT_GET_DATA=               -2023;
    LTR212_ERR_CANT_MANAGE_FILTERS=         -2024;
    LTR212_ERR_CANT_STOP=                   -2025;
    LTR212_ERR_CANT_GET_FRAME=              -2026;
    LTR212_ERR_INV_ADC_DATA=                -2026;
    LTR212_ERR_TEST_NOT_PASSED=             -2027;
    LTR212_ERR_CANT_WRITE_SERIAL_NUM =      -2028;
    LTR212_ERR_CANT_RESET_MODULE =          -2029;
    LTR212_ERR_MODULE_NO_RESPONCE =         -2030;
    LTR212_ERR_WRONG_FLASH_CRC=             -2031;
    LTR212_ERR_CANT_USE_FABRIC_AND_USER_CALIBR_SYM=-2032;
    LTR212_ERR_CANT_START_INTERFACE_TEST=   -2033;
    LTR212_ERR_WRONG_BIOS_FILE=             -2034;
    LTR212_ERR_CANT_USE_CALIBR_MODE=        -2035;
    LTR212_ERR_PARITY_ERROR   =             -2036;
    LTR212_ERR_CANT_LOAD_CLB_COEFFS =       -2037;
    LTR212_ERR_CANT_LOAD_FABRIC_CLB_COEFFS =-2038;
    LTR212_ERR_CANT_GET_VER=                -2039;
    LTR212_ERR_CANT_GET_DATE=               -2040;
    LTR212_ERR_WRONG_SN =                   -2041;
    LTR212_ERR_CANT_EVAL_DAC=               -2042;
    LTR212_ERR_ERROR_OVERFLOW =             -2043;
    LTR212_ERR_SOME_CHENNEL_CANT_CLB      = -2044;
    LTR212_ERR_CANT_GET_MODULE_TYPE                             =-2045;
    LTR212_ERR_ERASE_OR_PROGRAM_FLASH                           =-2046;
    LTR212_ERR_CANT_SET_BRIDGE_CONNECTIONS                      =-2047;
    LTR212_ERR_CANT_SET_BRIDGE_CONNECTIONS_FOR_THIS_MODULE_TYPE =-2048;
    LTR212_ERR_QB_RESISTORS_IN_ALL_CHANNELS_MUST_BE_EQUAL       =-2049;

    LTR212_ERR_INVALID_EEPROM_ADDR                              =-2050;
    LTR212_ERR_INVALID_VCH_CNT                                  =-2051;
    LTR212_ERR_FILTER_FILE_OPEN                                 =-2052;
    LTR212_ERR_FILTER_FILE_READ                                 =-2053;
    LTR212_ERR_FILTER_FILE_FORMAT                               =-2054;
    LTR212_ERR_FILTER_ORDER                                     =-2055;
    LTR212_ERR_UNSUPPORTED_MODULE_TYPE                          =-2056;



    LTR212_LCH_CNT_MAX      = 8;  // .  . 


    LTR212_FIR_ORDER_MAX    = 255; //    -
    LTR212_FIR_ORDER_MIN    = 3;   //    -


    //  
    LTR212_OLD = 0;   //     -  - 
    LTR212_M_1 = 1;   //     -,  -  - 
    LTR212_M_2 = 2;   //     -  - 

    //   
    LTR212_FULL_OR_HALF_BRIDGE                          = 0;
    LTR212_QUARTER_BRIDGE_WITH_200_Ohm                  = 1;
    LTR212_QUARTER_BRIDGE_WITH_350_Ohm                  = 2;
    LTR212_QUARTER_BRIDGE_WITH_CUSTOM_Ohm               = 3;
    LTR212_UNBALANCED_QUARTER_BRIDGE_WITH_200_Ohm       = 4;
    LTR212_UNBALANCED_QUARTER_BRIDGE_WITH_350_Ohm       = 5;
    LTR212_UNBALANCED_QUARTER_BRIDGE_WITH_CUSTOM_Ohm    = 6;

    //    (AcqMode)
    LTR212_ACQ_MODE_MEDIUM_PRECISION    = 0;
    LTR212_ACQ_MODE_HIGH_PRECISION      = 1;
    LTR212_ACQ_MODE_8CH_HIGH_PRECISION  = 2;



    //   
    LTR212_REF_2_5V = 0;  //2.5 
    LTR212_REF_5V   = 1;   //5   

    //  
    LTR212_SCALE_B_10 = 0; //  -10/+10
    LTR212_SCALE_B_20 = 1; //  -20/+20
    LTR212_SCALE_B_40 = 2; //  -40/+40
    LTR212_SCALE_B_80 = 3; //  -80/+80
    LTR212_SCALE_U_10 = 4; //  -10/+10
    LTR212_SCALE_U_20 = 5; //  -20/+20
    LTR212_SCALE_U_40 = 6; //  -40/+40
    LTR212_SCALE_U_80 = 7; //  -80/+80

    //  
    LTR212_CALIBR_MODE_INT_ZERO             = 0;
    LTR212_CALIBR_MODE_INT_SCALE            = 1;
    LTR212_CALIBR_MODE_INT_FULL             = 2;
    LTR212_CALIBR_MODE_EXT_ZERO             = 3;
    LTR212_CALIBR_MODE_EXT_SCALE            = 4;
    LTR212_CALIBR_MODE_EXT_ZERO_INT_SCALE   = 5;
    LTR212_CALIBR_MODE_EXT_FULL_2ND_STAGE   = 6; //    
    LTR212_CALIBR_MODE_EXT_ZERO_SAVE_SCALE  = 7; //        . 



 //********     *************/
type

{$A4}
    TINFO_LTR212 = record
        Name:       array[0..14] of AnsiChar;
        ModuleType: byte;
        Serial:     array[0..23] of AnsiChar;
        BiosVersion:array[0..7]  of AnsiChar; //  
        BiosDate:   array[0..15] of AnsiChar;//     
    end;

    TLTR212_Filter = record
        IIR:integer;         //   -
        FIR:integer;         //   -
        Decimation:integer;  //     -
        TAP:integer;         //  -
        IIR_Name:array[0..512] of AnsiChar; //      -  -
        FIR_Name:array[0..512] of AnsiChar; //      -  -
    end;

    TLTR212_Usr_Clb = record
        Offset   :array[0..LTR212_LCH_CNT_MAX-1]of Cardinal;
        Scale    :array[0..LTR212_LCH_CNT_MAX-1]of Cardinal;
        DAC_Value:array[0..LTR212_LCH_CNT_MAX-1]of byte;
    end;

    TLTR212 = record
        size:integer;
        Channel:TLTR;
        AcqMode:integer;     //   
        UseClb:integer;      //    -
        UseFabricClb:integer;//     -
        LChQnt:integer;      // -   
        LChTbl:array[0..LTR212_LCH_CNT_MAX-1]of integer;  //  
        REF:integer;         //    
        AC:integer;          //    
        Fs:double;           //   

        filter:TLTR212_Filter;

        ModuleInfo:TINFO_LTR212;
        CRC_PM:Word; //   
        CRC_Flash_Eval:Word; //   
        CRC_Flash_Read:Word;   //   
    end;

    pTLTR212 = ^TLTR212;

    ltr212filter = record
       fs:double;
       decimation:byte;
       taps:byte;
       koeff:array[0..LTR212_FIR_ORDER_MAX-1]of Smallint;
    end;
{$A+}

//  
Function LTR212_Init(module:pTLTR212):Integer; stdcall;
Function LTR212_IsOpened(module:pTLTR212):Integer;stdcall;
Function LTR212_Open(module:pTLTR212; net_addr:Cardinal; net_port:WORD; crate_snCHAR:Pointer; slot_num:integer; biosnameCHAR:Pointer):Integer;stdcall;
Function LTR212_Close(module:pTLTR212):Integer;stdcall;
Function LTR212_CreateLChannel(PhysChannel:integer; Scale:integer):Integer;stdcall;
Function LTR212_CreateLChannel2(PhysChannel:Cardinal; Scale:Cardinal; BridgeType:Cardinal):Integer; stdcall;

Function LTR212_SetADC(module:pTLTR212):Integer;stdcall;
Function LTR212_Start(module:pTLTR212):Integer;stdcall;
Function LTR212_Stop(module:pTLTR212):Integer;stdcall;
Function LTR212_Recv(module:pTLTR212; dataDWORD:Pointer; tmarkDWORD:Pointer; size:Cardinal; timeout:Cardinal):Integer;stdcall;
Function LTR212_ProcessData(module:pTLTR212; srcDWORD:Pointer;  destDOUBLE:Pointer; sizeDWORD:Pointer; volt:Boolean):Integer;stdcall;
Function LTR212_Calibrate(module:pTLTR212; LChannel_MaskBYTE:pointer; mode:integer; reset:integer):Integer;stdcall;
Function LTR212_CalcFS(module:pTLTR212; fsBaseDOUBLE:pointer; fs:pointer):Integer;stdcall;
Function LTR212_TestEEPROM(module:pTLTR212):Integer;stdcall;
//  
Function LTR212_ProcessDataTest(module:pTLTR212; srcDWORD:pointer;  destDOUBLE:Pointer; sizeDWORD:pointer; volt:boolean; bad_numDWORD:pointer):Integer;stdcall;
Function LTR212_ReadFilter(fnameCHAR:Pointer; filter_ltr212filter:Pointer):Integer;stdcall;
Function LTR212_WriteSerialNumber(module:pTLTR212; snCHAR:pointer; Code:WORD):Integer;stdcall;
Function LTR212_TestInterfaceStart(module:pTLTR212; PackDelay:integer):Integer;stdcall;
Function LTR212_CalcTimeOut(module:pTLTR212; n:Cardinal):Cardinal;stdcall;

Function LTR212_ReadUSR_Clb (module:pTLTR212; CALLIBR:pointer):Integer;stdcall;
Function LTR212_WriteUSR_Clb(module:pTLTR212; CALLIBR:pointer):Integer;stdcall;

Function LTR212_GetErrorString(err:integer):string;stdcall;


implementation
  Function LTR212_Init(module:pTLTR212):Integer; external 'ltr212api.dll';
  Function LTR212_IsOpened(module:pTLTR212):Integer;external 'ltr212api.dll';
  Function LTR212_Open(module:pTLTR212; net_addr:Cardinal; net_port:WORD; crate_snCHAR:Pointer; slot_num:integer; biosnameCHAR:Pointer):Integer;external 'ltr212api.dll';
  Function LTR212_Close(module:pTLTR212):Integer;external 'ltr212api.dll';
  Function LTR212_CreateLChannel(PhysChannel:integer; Scale:integer):Integer;external 'ltr212api.dll';
  Function LTR212_CreateLChannel2(PhysChannel:Cardinal; Scale:Cardinal; BridgeType:Cardinal):Integer;external 'ltr212api.dll';
  Function LTR212_SetADC(module:pTLTR212):Integer;external 'ltr212api.dll';
  Function LTR212_Start(module:pTLTR212):Integer;external 'ltr212api.dll';
  Function LTR212_Stop(module:pTLTR212):Integer;external 'ltr212api.dll';
  Function LTR212_Recv(module:pTLTR212; dataDWORD:Pointer; tmarkDWORD:Pointer; size:Cardinal; timeout:Cardinal):Integer;external 'ltr212api.dll';
  Function LTR212_ProcessData(module:pTLTR212; srcDWORD:Pointer;  destDOUBLE:Pointer; sizeDWORD:Pointer; volt:Boolean):Integer;external 'ltr212api.dll';
  Function LTR212_Calibrate(module:pTLTR212; LChannel_MaskBYTE:pointer; mode:integer; reset:integer):Integer;external 'ltr212api.dll';
  Function LTR212_CalcFS(module:pTLTR212; fsBaseDOUBLE:pointer; fs:pointer):Integer;external 'ltr212api.dll';
  Function LTR212_TestEEPROM(module:pTLTR212):Integer;external 'ltr212api.dll';
  //  
  Function LTR212_ProcessDataTest(module:pTLTR212; srcDWORD:pointer;  destDOUBLE:Pointer; sizeDWORD:pointer; volt:boolean; bad_numDWORD:pointer):Integer;external 'ltr212api.dll';
  Function LTR212_ReadFilter(fnameCHAR:Pointer; filter_ltr212filter:Pointer):Integer;external 'ltr212api.dll';
  Function LTR212_WriteSerialNumber(module:pTLTR212; snCHAR:pointer; Code:WORD):Integer;external 'ltr212api.dll';
  Function LTR212_TestInterfaceStart(module:pTLTR212; PackDelay:integer):Integer;external 'ltr212api.dll';
  Function LTR212_CalcTimeOut(module:pTLTR212; n:Cardinal):Cardinal;external 'ltr212api.dll';

  Function LTR212_ReadUSR_Clb (module:pTLTR212; CALLIBR:pointer):Integer;external 'ltr212api.dll';
  Function LTR212_WriteUSR_Clb(module:pTLTR212; CALLIBR:pointer):Integer;external 'ltr212api.dll';

  Function _get_err_str(err : integer) : PAnsiChar; stdcall; external 'ltr212api.dll' name 'LTR212_GetErrorString';

  Function LTR212_GetErrorString(err: Integer) : string; stdcall;
  begin
     LTR212_GetErrorString:=string(_get_err_str(err));
  end;
end.
