#ifndef LTRSRV_USB_P_H
#define LTRSRV_USB_P_H

#include "ltrsrv_crates.h"
#include <stdio.h>

#define LTRSRV_USB_CTL_TOUT                     1500

#define LTRSRV_USB_BULK_MAX_TRANSF_CNT          20
#define LTRSRV_USB_BULK_TRANSF_SIZE             (32*1024)
#define LTRSRV_USB_BULK_SEND_TOUT               0
#define LTRSRV_USB_READ_FPGA_INFO_TOUT          1000
#define LTRSRV_USB_READ_READ_TOUT               0
#define LTRSRV_USB_DEVCHECK_TOUT                500


#define USB_INIT_DROP_TRANSF_LIMIT             4096


/* при данном кол-ве ошибок перестаем делать попытки повторного подключения */
#define LTRSRV_USB_RECONNECT_ERR_TRESHOLD       4



/* проверка провильности описателя крейта */
#define USB_CHECK_HND(hnd) (hnd) == NULL ? LTRSRV_ERR_CRATE_HANDLE : \
    (hnd)->intf_data == NULL ? LTRSRV_ERR_CRATE_INTF_CLOSED : \
    (hnd)->par.intf != LTR_CRATE_IFACE_USB ? LTRSRV_ERR_CRATE_INVALID_INTF : 0

typedef enum {
    LTR_USB_CMD_RESET_FPGA              = 0,
    LTR_USB_CMD_PUT_ARRAY               = 1,
    LTR_USB_CMD_GET_ARRAY               = 2,
    LTR_USB_CMD_INIT_FPGA               = 3,
    LTR_USB_CMD_SET_DMA_PARAMETRS       = 4,
    LTR_USB_CMD_GET_MODULE_NAME         = 11,
    LTR_USB_CMD_SET_WAIT_TIME           = 12,
    LTR_USB_CMD_BOOTLOADER_PUT_ARRAY    = 13,
    LTR_USB_CMD_BOOTLOADER_GET_ARRAY    = 14,
    LTR_USB_CMD_CALL_APPLICATION        = 15
} t_ltr_crate_usb_cmds;

typedef enum {
    USB_INIT_GET_NAME = 0,
    USB_INIT_GET_DESCR = 1,
    USB_INIT_GET_BOOTLOADER_VER = 2,
    USB_INIT_GET_FIRM_VER = 3,
    USB_INIT_GET_HARD_REV = 4,
    USB_INIT_DMAC_INIT_0  = 5,
    USB_INIT_FPGA_RESET   = 6,
    USB_INIT_FPGA_INIT    = 7,
    USB_INIT_DMAC_INIT_1  = 8,
    USB_INIT_FPGA_READ_INFO = 9,
    USB_INIT_SWITCH_BOOTLOADER = 10,
    USB_INIT_SWITCH_APPLICATION = 11,
    USB_INIT_FPGA_LOAD = 12,
    USB_INIT_FPGA_LOAD_DONE = 13,
    USB_INIT_DONE = 0xFE,
    USB_INIT_CLEANUP = 0xFF
} t_init_state;


typedef enum {
    USB_INTF_FLAGS_SPECIAL_FPGA_DATA_PROC = 0x01,
    USB_INTF_FLAGS_LOAD_FPGA              = 0x02,
    USB_INIT_FLAGS_SWITCH_BOOTLOADER      = 0x04
} t_usb_intf_flags;

typedef enum {
    USB_BULK_DIR_READ  = 0,
    USB_BULK_DIR_WRITE = 1
} t_usb_bulk_dir;


typedef enum {
    USB_TRANSF_STATE_FREE        = 0,
    USB_TRANSF_STATE_PROGRESS    = 1,
    USB_TRANSF_STATE_CLEANUP     = 2
} t_usb_transf_state;


typedef struct {
    void* port_transfer;
    uint8_t *buf;
    uint16_t len;
    t_usb_transf_state state;
} t_transf_params;

typedef struct {
    t_transf_params transfs[LTRSRV_USB_BULK_MAX_TRANSF_CNT];
    uint8_t put_pos;
    uint8_t get_pos;
    t_usb_bulk_dir dir;
} t_bulk_transf_ctl;


struct st_usb_port_intf;

typedef struct st_usb_data {
    int fnd; /* признак, найдено ли это устройство при очередной проверке (для
                нахождения отключенных устройств) */
    void *devid; /* параметр одназначно опеределяющий устройство */
    t_init_state init_state; /* Состояние автомата инициализации крейта */
    int flags; /* Флаги из #t_usb_intf_flags */
    t_ltr_crate* crate; /* Указатель на описатель крейта */

    t_ltr_crate_cmd_cb proc_cmd_cb; /* callback верхнего уровня на завершение
                                       выполнение команды */
    void* proc_data;  /* указатель, который передается proc_cmd_cb */
    uint8_t* proc_buf; /* буфер, в который принимается ответ на упр. запро*/
    uint16_t proc_len; /* размер буфера на прием ответа на упр. запрос */
    uint32_t* proc_rcv_size; /* укзатель, куда нужно сохранить принятый размер
                                данных от запроса, если он может отличаться
                                от запрошенного */


    t_bulk_transf_ctl bulk_rd;
    t_bulk_transf_ctl bulk_wr;

    t_transf_params ctl;

    t_ltr_crate_snd_rdy_cb snd_rdy_cb;


    FILE* firm_file; /* Файл с прошивкой крейта */
    const char* firm_file_name; /* имя файла с прошивкой (если нужно загружать) */
    uint32_t fpga_load_size;
    char* fpga_info;
    uint16_t fpga_cur_info_size;
    uint16_t fpga_info_req_size;
    int drop_init_wrds;
    int drop_exception;

    const struct st_usb_port_intf* port_intf;
    void *privdata;
} t_usb_data;





typedef struct st_usb_port_intf {
    void (*fill_location)(t_usb_data *intf);
    int  (*start_ioctl)(t_usb_data* intf, uint8_t cmd, uint32_t param,
                       uint16_t reply_len, uint16_t data_len,
                       const uint8_t* tx_data);
    int  (*start_bulk)(t_usb_data* intf, t_bulk_transf_ctl* bulk_ctl, unsigned timeout);
    void (*cancel_transfer)(t_usb_data *intf, t_transf_params *transf);
    void (*cleanup)(t_usb_data *intf);
    FILE* (*fopen)(const char* fname, const char* mode);


    int   (*devid_cmp)(void *dev1, void *dev2);
    void* (*devid_create)(void *dev);
    void  (*devid_free)(void *dev);
} t_usb_port_intf;


typedef struct {
    t_ltr_crate_type type;
    uint16_t vid;
    uint16_t pid;
} t_usb_dev_id;

extern t_usb_data* f_usb_crates[];
extern unsigned f_usb_crates_cnt;
extern const t_usb_dev_id crate_usb_dev_ids[];

/* Функция должна вызываться перед началом получения списка устройств */
void p_usb_check_devlist_init(void);
/* Функция должна вызываться после завершения получения списка устройств */
void p_usb_check_devlist_finish(void);
/* проверка, является ли устройство с указанным признаком новым (т.е. требуется
 *  его инициализация) */
int  p_usb_check_dev_new(void *devid, const t_usb_port_intf *port_intf);

int p_usb_add_new_dev(void *devid, void *privdata, const t_usb_port_intf *port_intf, t_usb_data **pintf);

int p_usb_process_ioctl(t_usb_data *intf, int transf_err, const uint8_t* replay, unsigned actual_len);
int p_usb_process_bulk_rd(t_usb_data *intf, int transf_err, const uint8_t* data, unsigned len);
int p_usb_process_bulk_wr(t_usb_data *intf, int transf_err, uint8_t* data, unsigned len);

void p_usb_set_deverr(void *dev, const t_usb_port_intf *port_intf);
void p_usb_free_data(t_usb_data* intf);
void p_usb_free_dev_by_err(t_usb_data* intf);
void p_usb_cleanup_finish(t_usb_data *intf) ;



#endif // LTRSRV_USB_P_H
