#include "CrateConfigDialog.h"
#include "ui_CrateConfigDialog.h"

#include <QMessageBox>
#include <QPushButton>



#ifdef Q_OS_WIN
    #define QSTRING_FROM_CSTR(str) QString::fromLocal8Bit(str)
#else
    #define QSTRING_FROM_CSTR(str) QString(str)
#endif

static int f_if_tbl[] = {LTR_CRATE_IFACE_USB, LTR_CRATE_IFACE_TCPIP, LTR_CRATE_IFACE_UNKNOWN};

CrateConfigDialog::CrateConfigDialog(LtrSrvConnection* con, LtrCrateInfo *info, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CrateConfigDialog) {
    ui->setupUi(this);

    m_applyBtn = ui->buttonBox->button(QDialogButtonBox::Apply);
    QString ipNubReg = "(25\\d|2[0-4]{1,1}\\d|[0-1]{0,1}\\d{1,2}|\\d{1,2}){1,1}";
    QRegExpValidator* validator = new QRegExpValidator(
                QRegExp( ipNubReg + "\\." + ipNubReg + "\\."
                         + ipNubReg + "\\." + ipNubReg), this);
    ui->CrateIpAddr->setValidator(validator);
    ui->CrateIpMask->setValidator(validator);
    ui->CrateGateway->setValidator(validator);
    connect(ui->CrateIpAddr, SIGNAL(textChanged(QString)), SLOT(dataChanged(void)));
    connect(ui->CrateIpMask, SIGNAL(textChanged(QString)), SLOT(dataChanged(void)));
    connect(ui->CrateGateway, SIGNAL(textChanged(QString)), SLOT(dataChanged(void)));
    connect(ui->CrateInterface, SIGNAL(currentIndexChanged(int)), SLOT(dataChanged()));

    connect(m_applyBtn, SIGNAL(clicked()), SLOT(apply()));


    LTR030_Init(&m_hnd);
    INT err = LTR030_Open(&m_hnd, con->addr().toIPv4Address(), con->port(), info->crateInterface(), info->serial().toAscii());
    if (err==LTR_OK) {
        TLTR030_CONFIG cfg;
        cfg.Size = sizeof(cfg);
        err = LTR030_GetConfig(&m_hnd, &cfg);
        if (err==LTR_OK) {
            int pos= sizeof(f_if_tbl)/sizeof(f_if_tbl[0])-1;
            for (unsigned i=0; i < sizeof(f_if_tbl)/sizeof(f_if_tbl[0]); i++) {
                if (f_if_tbl[i]==cfg.Interface)
                    pos = i;
            }


            ui->CrateInterface->setCurrentIndex(pos);
            ui->CrateIpAddr->setText(QHostAddress(cfg.IpAddr).toString());
            ui->CrateIpMask->setText(QHostAddress(cfg.IpMask).toString());
            ui->CrateGateway->setText(QHostAddress(cfg.Gateway).toString());

            dataChanged();
            m_modified = false;
            m_applyBtn->setEnabled(false);
        } else {
            QMessageBox::critical(this, tr("Getting crate configuration error!"),
                                  QSTRING_FROM_CSTR(LTR030_GetErrorString(err)),
                                  QMessageBox::Ok, QMessageBox::Ok);
            this->reject();
        }
    } else {
        QMessageBox::critical(this, tr("Cannot open crate connection!"),
                              QSTRING_FROM_CSTR(LTR030_GetErrorString(err)),
                              QMessageBox::Ok, QMessageBox::Ok);
        this->reject();
    }
}

CrateConfigDialog::~CrateConfigDialog() {
    delete ui;
}

void CrateConfigDialog::done(int ret) {
    LTR030_Close(&m_hnd);
    QDialog::done(ret);
}

void CrateConfigDialog::accept() {
    if (m_modified) {
        if (QMessageBox::information(this, tr("Unsaved changes"),
                                     tr("Unsaved changes detected! Save them?"),
                                     QMessageBox::Yes | QMessageBox::No)==QMessageBox::Yes) {
            if (apply()==LTR_OK) {
                QDialog::accept();
            }
        } else {
            QDialog::accept();
        }
    } else {
        QDialog::accept();
    }
}

void CrateConfigDialog::dataChanged() {
    bool cfg_ok = true;
    int pos=0;
    m_modified = true;

    QString text = ui->CrateIpAddr->text();
    if (ui->CrateIpAddr->validator()->validate(text, pos) == QValidator::Acceptable) {
        ui->CrateIpAddr->setStyleSheet("QLineEdit { color: darkGreen }");
    } else {
        ui->CrateIpAddr->setStyleSheet("QLineEdit { color: darkRed }");
        cfg_ok = false;
    }

    text = ui->CrateIpMask->text();
    if (ui->CrateIpMask->validator()->validate(text, pos) == QValidator::Acceptable) {
        ui->CrateIpMask->setStyleSheet("QLineEdit { color: darkGreen }");
    } else {
        ui->CrateIpMask->setStyleSheet("QLineEdit { color: darkRed }");
        cfg_ok = false;
    }

    text = ui->CrateGateway->text();
    if (ui->CrateGateway->validator()->validate(text, pos) == QValidator::Acceptable) {
        ui->CrateGateway->setStyleSheet("QLineEdit { color: darkGreen }");
    } else {
        ui->CrateGateway->setStyleSheet("QLineEdit { color: darkRed }");
        cfg_ok = false;
    }

    if (f_if_tbl[ui->CrateInterface->currentIndex()]==LTR_CRATE_IFACE_UNKNOWN)
        cfg_ok =  false;

    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(cfg_ok);
    m_applyBtn->setEnabled(cfg_ok);
}

int CrateConfigDialog::apply() {
    TLTR030_CONFIG cfg;
    cfg.Size = sizeof(cfg);
    cfg.Interface = f_if_tbl[ui->CrateInterface->currentIndex()];
    cfg.IpAddr = QHostAddress(ui->CrateIpAddr->text()).toIPv4Address();
    cfg.IpMask = QHostAddress(ui->CrateIpMask->text()).toIPv4Address();
    cfg.Gateway = QHostAddress(ui->CrateGateway->text()).toIPv4Address();

    INT err = LTR030_SetConfig(&m_hnd, &cfg);
    if (err==LTR_OK) {
        m_modified = false;
        m_applyBtn->setEnabled(false);

        QMessageBox::information(this, tr("Operation completed"),
                              tr("Crate settings was successfully set! You must turn off and on crate for new settings to take effect!"));
    } else {
        QMessageBox::critical(this, tr("Cannot set crate settings!"),
                              QSTRING_FROM_CSTR(LTR030_GetErrorString(err)));
    }
    return err;
}
