#include "LQTableWidget.h"
#include <QKeyEvent>
#include <QApplication>
#include <QClipboard>

LQTableWidget::LQTableWidget(QWidget *parent) :
    QTableWidget(parent)
{

}

LQTableWidget::LQTableWidget(int rows, int columns, QWidget *parent)
    : QTableWidget(rows, columns, parent)
{
}

void LQTableWidget::keyPressEvent(QKeyEvent *event)
{
    if ((event->key()==Qt::Key_C) && (event->modifiers()==Qt::ControlModifier))
    {

        copySelectionToClipboard();
        event->accept();
    }
    else
    {
        QTableWidget::keyPressEvent(event);
    }
}

void LQTableWidget::copySelectionToClipboard()
{
    QItemSelectionModel * selection = this->selectionModel();
    QModelIndexList indexes = selection->selectedIndexes();
    qSort(indexes.begin(), indexes.end());
    if (!indexes.isEmpty())
    {
        QString selected_text;
        QModelIndex previous = indexes.first();
        QAbstractItemModel * model = this->model();
        indexes.removeFirst();




        foreach(QModelIndex current, indexes)
        {
            if (!isIndexHidden(current)) {
                QVariant data = model->data(previous);
                QString text = data.toString();
                // At this point `text` contains the text in one cell
                selected_text.append(text);
                // If you are at the start of the row the row number of the previous index
                // isn't the same.  Text is followed by a row separator, which is a newline.
                if (current.row() != previous.row())
                {
                    selected_text.append('\n');
                }
                // Otherwise it's the same row, so append a column separator, which is a tab.
                else
                {
                    selected_text.append('\t');
                }
                previous = current;
            }
        }

        // At this point `text` contains the text in one cell
        selected_text.append(model->data(previous).toString());

        QApplication::clipboard()->setText(selected_text);
    }
}
